#!/usr/bin/perl
use strict;

package MATT::Mysql;

=head1 NAME

	MATT::Mysql - Common Mysql functions

=head1 SYNOPSIS

	MATT::Mysql is a grouping of frequently used functions I've written
	for Mysql.

=head1 DESCRIPTION

	I find myself using MySQL for a lot of things. Geographically distributed
	dns systems (MySQL replication), mail servers, and all the other fun stuff
	you'd use a RDBMS for. As such, I've got a growing pile of scripts that
	have lots of duplicated code in them. As such, the need for this Perl
	module grew.

=cut

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

$VERSION  = '0.6.0';

use Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
	MysqlConnect 
	MysqlQuery 
	ParseDotFile 
	SetAutocommit 
	MysqlDBVars
	MysqlSanity
	MysqlVersion
	IsMysqlNewer
	MysqlLockTables
	MysqlUnlockTables
	InstallMysql
);
@EXPORT_OK = qw( InstallMysqlTool );

##
# Scripts that use this library (used in)
##
# mysql_replicate_manager v1.5+
# uron.net user_*.pl
# polls.pl
# nt_export_djb_update.pl
# toaster_setup.pl 

use Carp;
use Getopt::Std;
use POSIX qw(uname);
use lib ".";
use lib "lib";
use MATT::Perl;

LoadModule("DBI",        "p5-DBI",       "databases", 1);
LoadModule("DBD::mysql", "p5-DBD-mysql", "databases", 1);

sub MysqlQuery($$;$)
{
	my ($dbh, $query, $warn) = @_;

=head2 MysqlQuery

	my $sth = MysqlQuery ($dbh, $query, $warn)

	$dbh is the database handle you've already acquired via MysqlConnect.

	$query is the SQL statement to execute.

	If $warn is set, we don't die if the query fails. This way you can
	decide when you call the sub whether you want it to die or return
	a failed $sth (and likely an error message).

	execute performs anything necessary to execute a prepared statement
	Always returns true regardless of # of rows affected.
	For non-Select, returns # of rows affected: No rows = 0E0
	For Select, simply starts query. Follow with fetch_*

=cut

	if ($warn)
	{
		if ( my $sth = $dbh->prepare($query) ) 
		{
			$sth->execute or carp "couldn't execute: $sth->errstr\n";
			#$dbh->commit  or warn "couldn't commit: $sth->errstr\n";
			return $sth;
		} 
		else
		{
			warn "couldn't prepare: $DBI::errstr\n";
			return $sth;
		};
	} 
	else 
	{
		if ( my $sth = $dbh->prepare($query) ) 
		{
			$sth->execute or croak "couldn't execute: $sth->errstr\n";
			#$dbh->commit  or croak "couldn't commit: $sth->errstr\n";
			return $sth;
		} 
		else 
		{
			croak "couldn't prepare: $DBI::errstr\n";
		};
	};
};

sub MysqlConnect($;$$)
{

=head2 MysqlConnect

	use MATT::Mysql;
	my ($dbh, $dsn, $drh) = MysqlConnect($dot, $warn, $debug);

	$dot is a hashref of key/value pairs in the same format you'd find in
	~/.my.cnf. No coincidentally, that's where it expects you'll be
	getting them from.

	$warn allows you to determine whether to die or warn on failure or
	error. To warn, set $warn to a non-zero value. 

	$debug will print out helpful debugging messages should you be having
	problems.

=cut

	my ( $dot, $warn, $debug) = @_;
	my $dbh;

	my $ac  = SetAutocommit( $dot );
	my $dbv = MysqlDBVars  ( $dot );
	my $dsn = "DBI:$dbv->{'driver'}:database=$dbv->{'db'};host=$dbv->{'host'};port=$dbv->{'port'}";

	if ($warn) 
	{
		$dbh = DBI->connect($dsn, $dbv->{'user'}, $dbv->{'pass'},
			{ RaiseError => 0, AutoCommit => $ac } )
				or carp "db connect failed: $!\n";
	} 
	else 
	{
		$dbh = DBI->connect($dsn, $dbv->{'user'}, $dbv->{'pass'},
			{ RaiseError => 0, AutoCommit => $ac } ) 
				or croak "db connect failed: $!\n";
	};
	my $drh = DBI->install_driver($dbv->{'driver'});

	return ($dbh, $dsn, $drh);
};

sub MysqlDBVars($)
{
	my ($val) = @_;
	my ($driver, $db, $host, $port, $user, $pass, $dir);

=head2 MysqlDBVars

	This sub is called internally by MysqlConnect and is used principally 
	to set some reasonable defaults should you not pass along enough 
	connection parameters in $dot. 

=cut

	if ( $val->{'driver'} && $val->{'driver'} ne "" ) 
	{    $driver= $val->{'driver'} } else { $driver= "mysql" };

	if ( $val->{'db'}     && $val->{'db'}     ne "" ) 
	{    $db    = $val->{'db'}     } else { $db    = "mysql" };

	if ( $val->{'host'}   && $val->{'host'}   ne "" ) 
	{    $host  = $val->{'host'}   } else { $host  = "localhost" };

	if ( $val->{'port'}   && $val->{'port'}   ne "" ) 
	{    $port  = $val->{'port'}   } else { $port  = "3306" };

	if ( $val->{'user'}   && $val->{'user'}   ne "" ) 
	{    $user  = $val->{'user'}   } else { $user  = "root" };

	if ( $val->{'pass'}   && $val->{'pass'}   ne "" ) 
	{    $pass  = $val->{'pass'}   } else { $pass  = "" };

	if ( $val->{'dir_m'}  && $val->{'dir_m'}  ne "" ) 
	{    $dir ="/var/db/mysql"} else { $dir   = $val->{'dir_m'} };     
  
	my %master = ( driver  => $driver,     db      => $db,      host    => $host,
			port    => $port,       user    => $user,    pass    => $pass,
			dir     => $dir );
	return \%master;
};

sub SetAutocommit 
{
	my ($dot) = @_;

	if ($dot->{'autocommit'} && $dot->{'autocommit'} ne "" ) 
	{
		return $dot->{'autocommit'};  #	SetAutocommit
	} 
	else 
	{
		return 1;                     #  Default to autocommit.
	};
};

sub ParseDotFile($$;$)
{
	my ($file, $start, $debug) = @_;

	my ($homedir)   = (getpwuid ($<))[7];
	my $dotfile     = "$homedir/$file";

=head2 ParseDotFile

	use MATT::Mysql;
	my $dot = ParseDotFile(".my.cnf", "[mysql_replicate_manager]", 0);


	ParseDotFile ($file, $start, $debug)

	$file is the file to be parsed.

	$start is the [identifier] where we begin looking for settings. 
	This expects the format used in .my.cnf MySQL configuration files.

	A hashref is returned wih key value pairs

=cut

	if (-r $dotfile) 
	{
		my (%array);
		my $gotit = 0;
	
		print "ParseDotFile: $dotfile\n" if ($debug);
		open(DOT, $dotfile) or carp "WARNING: Can't open $dotfile: $!";
		while ( <DOT> ) 
		{
			next if /^#/;
			my $line = $_; chomp $line;
			if ( !$gotit ) 
			{
				print "1. $line\n" if $debug;
				if ($line eq $start) 
				{
					$gotit = 1; 
					next;
				};
			} 
			else 
			{
				if ( $line =~ /^\[/ ) { last };
				print "2. $line\n" if $debug;
				$line =~ /(\w+)\s*=\s*(.*)\s*$/;
				$array{$1} = $2 if $1;
			};
		};
	
		if ($debug) 
		{
			foreach my $key ( keys %array ) 
			{
				print "hash: $key\t=$array{$key}\n";
			};
		};
		close(DOT);
		return \%array;
	} else 
	{
		carp "WARNING: ParseDotFile: can't read $dotfile!\n";
		return 0;
	};
};

sub MysqlSanity($)
{

=head2	MysqlSanity

	A place to do validation tests on values to make sure they're reasonable

	Currently we only check to assure the password is less than 32 characters
	and the username is less than 16. Many more tests will come.

=cut

	my ($dot) = @_;

	if ( $dot->{'user'} )
	{
		if ( length($dot->{'user'}) > 16 )
		{
			croak "\n\nUsername cannot exceed 16 characters. Edit user in ~/.my.cnf\n\n"
		};
	}
	else
	{ 
		croak "\n\nYou have not configured ~/.my.cnf. Read the FAQ before proceeding.\n\n";
	};

	if ( $dot->{'pass'} )
	{
		if ( length($dot->{'pass'}) > 32 )
		{
			croak "\nPassword cannot exceed 16 characters. Edit pass in ~/.my.cnf\n\n"
		};
	}
	else
	{
		croak "\nYou have not configured ~/.my.cnf properly. Read the FAQ before proceeding.\n\n";
	};
};

sub IsMysqlNewer($$)
{

=head2	IsMysqlNewer

	use Matt::Mysql;

	my $ver   = MysqlVersion($dbh);
	my $newer = IsMysqlNewer("4.1.0", $ver);

	if ($newer) { print "you are brave!" };

	As you can see, IsMysqlNewer can be very useful, especially
	when you need to execute queries with syntax differences between
	versions of Mysql.

=cut

	my ($min, $cur) = @_;

	$min =~ /^([0-9]+)\.([0-9]{1,})\.([0-9]{1,})$/;
	my @mins = ( $1, $2, $3 );
	$cur =~ /^([0-9]+)\.([0-9]{1,})\.([0-9]{1,})$/;
	my @curs = ( $1, $2, $3 );

	if ( $curs[0] > $mins[0] ) { return 1; };
	if ( $curs[1] > $mins[1] ) { return 1; };
	if ( $curs[2] > $mins[2] ) { return 1; };

	return 0;
};

sub MysqlVersion($)
{

=head2	MysqlVersion
	
	use MATT::Mysql;
	my $ver = MysqlVersion($dbh);

	Returns a string representing the version of MySQL running.

=cut

	my ($dbh) = @_;
	my ($sth, $minor);

	if ( $sth = MysqlQuery($dbh, "SELECT VERSION()") )
	{
		my $r = $sth->fetchrow_arrayref;
		($minor) = split(/-/, $r->[0]);
		$sth->finish;
	};

	return $minor;
};

sub MysqlLockTables($;$)
{

=head2	MysqlLockTables

	use MATT::Mysql;
	MysqlLockTables($dbh, $debug);

	Takes a statement handle and does a global lock on all tables.
	Quite useful when you want do do things like make a tarball
	of the database directory, back up the server, etc.

=cut
	
	my ($dbh, $debug) = @_;

	# Table locking is done at the per-thread level. If we did a $sth->finish
	# the thread would end and we'd lose our lock. So, instead we pass the $sth
	# handle back and close it after we've done our deeds.

	print "lock_tables: locking tables.\n" if $debug;

	if (my $sth = MysqlQuery($dbh, "FLUSH TABLES WITH READ LOCK")) 
	{
		return $sth;
	};
};

sub MysqlUnlockTables($$;$)
{

=head2	MysqlUnlockTables

	use MATT::Mysql;
	MysqlUnlockTables($dbh, $sth, $debug);

	Takes a statement handle and does a global unlock on all tables.
	Quite useful after you've used MysqlLockTables, done your deeds
	and wish to return to normal.

=cut
	
	my ($dbh, $sth, $debug) = @_;

	print "UnlockTables: unlocking mysql tables.\n" if $debug;

	my $query = "UNLOCK TABLES";  # unnecessary, simply calling finish does this

	$sth = MysqlQuery($dbh, $query)
		or croak "FATAL: couldn't unlock tables: $sth->errstr\n";

	$sth->finish;
}

sub InstallMysql
{
	my ($mysql, $site) = @_;

	if ( (uname)[0] eq "FreeBSD" ) {

		my $ans = MATT::Utility::GetAnswer("You have several choices for installing Mysql:

	1. Install ? version from FreeBSD package.

        This will (very quickly) install whichever version of MySQL was released
         with the version of FreeBSD you are running. This quite often is not 
        what you want.

        2. Install Mysql 3.23.x, which is the latest MySQL in the 3.x branch.

        3. Install Mysql 4.0.x, which is the latest MySQL in the 4.x branch.

        4. Install Mysql 4.1.x, which is currently alpha quality. Not recommended.
");

		if ($ans == 1) 
		{
			InstallPackage("mysql-server", "databases");
			InstallPackage("mysql-client", "databases");
		} 
		elsif ($ans == 2) 
		{
			InstallPort("mysql323-server", "databases", "mysql323-server", "mysql-server-3");
			InstallPort("mysql323-client", "databases", "mysql323-client", "mysql-client-3");
		} 
		elsif ($ans == 3) 
		{
			InstallPort("mysql40-server", "databases", "mysql40-server", "mysql-server-4.0");
			InstallPort("mysql40-client", "databases", "mysql40-client", "mysql-client-4.0");
		} 
		elsif ($ans == 4) 
		{
			InstallPort("mysql41-server", "databases", "mysql41-server", "mysql-server-4.1");
			InstallPort("mysql41-client", "databases", "mysql41-client", "mysql-client-4.1");
		};

		InstallPort("p5-DBI",            "databases");
		InstallPort("p5-DBD-mysql",      "databases");

		#  This would be great to install but until it recognizes a previously
		#  installed mod_perl2, we can't use it via ports. :(
		#       InstallPort("p5-Apache-DBI",     "www");
	}
	elsif ( (uname)[0] eq "Darwin" ) 
	{
		$mysql = "mysql-standard-4.0.15" unless $mysql;
		$site  = "ftp://mysql.secsup.org/pub/software/mysql/Downloads/MySQL-4.0/" unless $site;

		chdir("~/Desktop/");

		unless ( -e "$mysql.tar.gz") {
			MATT::Utility::FetchFile("$site/$mysql.dmg");
		};

		print "There is a $mysql.dmg file in ~/Desktop/. Read and follow the readme
therein to install MySQL";	
	};
}


sub InstallMysqlTool($;$$)
{
	my ($package, $httpdir, $confdir) = @_;

	my $site = "http://www.dajoba.com/projects/mysqltool";
	$httpdir = "/usr/local/www" if (!$httpdir);
	$confdir = "/usr/local/etc" if (!$confdir);

	if ( (uname)[0] eq "FreeBSD" ) 
	{
		use MATT::FreeBSD;
		InstallPort("p5-Crypt-Blowfish", "security");
		InstallPort("p5-DBI",            "databases");
		InstallPort("p5-Apache-DBI",     "www");
		InstallPort("p5-DBD-mysql",      "databases");
		InstallPort("p5-SOAP-Lite",      "net");
	} 
	else 
	{
		use CPAN;
		install "Crypt::Blowfish";
		install "DBI";
		install "Apache::DBI";
		install "DBD::mysql";
		install "SOAP::Lite";
	};

	CdSrcDir("/usr/local/www");

	if (!-e "$package.tar.gz" ) 
	{
		FetchFile("$site/$package.tar.gz");
	};

	if ( -d "$package" )
	{
		my $r = SourceWarning($package, 1);
		if (! $r) { croak "sorry, I can't continue.\n"; };
	};

	my $tar = FindTheBin("tar");
	SysCmd("$tar -xzf $package.tar.gz");
	chdir($package);
	SysCmd( "perl Makefile.PL");
	SysCmd( "make");
	SysCmd( "make install clean");

	if ( !-e "$confdir/apache/mysqltool.conf" ) 
	{
		move("htdocs/mysqltool.conf", "$confdir/apache") or croak "move failed: $!\n";
	};

	if ( !-e "$httpdir/data/mysqltool" ) 
	{
		FetchFile("http://matt.simerson.net/computing/sql/mysqltool.index.txt");
		move("mysqltool.index.txt", "htdocs/index.cgi");
		move("htdocs", "$httpdir/data/mysqltool") or croak "move failed: $!\n";
	};
};



1;
__END__

=head1 Dependencies

	DBI.pm     - /usr/ports/databases/p5-DBI
	DBD::mysql - /usr/ports/databases/p5-DBD-mysql

	In order to use this module, you must have DBI.pm and DBD::Mysql
	installed. If they are not installed and you attempt to use this
	module, you should get some helpful error messages telling you how
	to install them.

=head1 AUTHOR

	Matt Simerson <matt@cadillac.net>

=head1 BUGS

	None known. Report any to author.

=head1 TODO

	Don't export any of the symbols by default. Move all symbols
	to EXPORT_OK and explicitely pull in the required ones in 
	programs that need them.

=head1 SEE ALSO

	http://www.tnpi.biz/internet/sql/

=head1 COPYRIGHT

	Copyright 2003, The Network People, Inc. All Rights Reserved.

=cut
