#!/usr/bin/perl

package MATT::Passwd;

=head1 NAME

	MATT::Passwd - Common Unix Passwd functions

=head1 SYNOPSIS

	MATT::Passwd is a grouping of frequently used functions I've written for 
	interacting with /etc/passwd entries.

=head1 DESCRIPTION

	Dependencies:

	Crypt::PasswdMD5 -  /usr/ports/security/p5-Crypt-PasswdMD5

=cut

use strict;
use Exporter;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

@ISA   = qw(Exporter);
@EXPORT = qw(
	UserSanity
	PasswordSanity
	DoesSysUserExist
	Encrypt
	BackupMasterPasswd
	VerifyMasterPasswd
	InstallGroup
	InstallUser
);
@EXPORT_OK = qw();
$VERSION = 0.8.3;

use Carp;
use POSIX qw(uname);
use lib ".";
use lib "lib";
use MATT::Perl;
use MATT::Utility;

sub UserSanity
{
	my ($user, $disallow) = @_;
	my %r = ( rc => '0' );

=head2 UserSanity

	use MATT::Passwd;

	$r = UserSanity($user, $denylist);   # $denylist is optional

	if ( $r->{'rc'} ) {  print "success"    }
	else              {  print $r->{'error' };


	$user is the username. Pass it along as a scalar (string).

	$denylist is a hashref. Optionally, define all usernames you want 
	reserved (denied) and it'll check to make sure $user is not in the
	hashref.

	Checks:

		* Usernames must be between 2 and 16 characters.
		* Usernames must have only lowercase alpha and numeric characters
		* Username must not be defined in $denylist hash
		* If the file /usr/local/etc/passwd.reserved exists, the username 
		must not exist in that file. 

	The format of passwd.reserved is simply one username per line.
	
	A hashref ($r) gets returned that will contain at least rc, and error. 

	$r->{'rc'}    will contain a result code of 0 (failure) or 1 (success). 
	$r->{'error'} will contain a string with a description of which test failed.

=cut

	# must have min 2 characters
	if ( length($user) < 2 )
	{
		$r{'error'} = "Usernames must have at least two characters. $user has less than two characters.";
		return \%r;
	};

	# max 16 characters
	if ( length($user) > 16 )
	{
		$r{'error'} = "Usernames must have no more than 16 characters. $user has more than 16 characters.";
		return \%r;
	};

	# only lower case letters and numbers
	unless  ( $user =~ /[a-z0-9]/ )
	{
		$r{'error'} = "Usernames must consist of only lower case letters and numbers. $user is not a valid username.";
		return \%r;
	};

	if ( $disallow )
	{
		if ( defined $disallow->{$user} ) {
			$r{'error'} = "$user is a reserved username. Please select another.";
			return \%r;
		};
	};

	if ( -r "/usr/local/etc/passwd.reserved" )
	{
		my @lines = MATT::Utility::ReadFile("/usr/local/etc/passwd.reserved");
		foreach my $line (@lines)
		{
			chomp $line;
			if ($user eq $line)
			{
				$r{'error'} = "$user is a reserved username. Please select another.";
				return \%r;
			};
		};
	};

	$r{'rc'} = 1;
	return \%r;
};

sub PasswordSanity
{
	my ($pass, $user) = @_;
	my %r = ( rc => '0' );

=head2 PasswordSanity

	use MATT::Passwd;

	$r =  PasswordSanity($password, $username);

	if ( $r->{'rc'} ) {  print "success"    }
	else              {  print $r->{'error' };


	$password  is the password the user is attempting to use.

	$username is the username the user has selected. 

	Checks: 

		Passwords must have at least 6 characters.
		Passwords must have no more than 128 characters.
		Passwords must not be the same as the username
		Passwords must not be purely alpha or purely numeric
		Passwords must not be in reserved list (/usr/local/etc/passwd.badpass)

	$r is a hashref that gets returned.

	$r->{'rc'}    will contain a result code of 0 (failure) or 1 (success). 
	$r->{'error'} will contain a string with a description of which test failed.

=cut

	# min 6 characters
	if ( length($pass) < 6 )
	{
		$r{'error'} = "Passwords must have at least six characters. $pass is too short.";
		return \%r;
	};

	# max 128 characters
	if ( length($pass) > 128 )
	{
		$r{'error'} = "Passwords must have no more than 128 characters. $pass is too long.";
		return \%r;
	};

	# not purely alpha or numeric
	if ( $pass =~ /a-z/ or $pass =~ /A-Z/ or $pass =~ /0-9/ )
	{
		$r{'error'} = "Passwords must contain both letters and numbers!";
		return \%r;
	};

	# does not match username
	if ( $pass eq $user )
	{
		$r{'error'} = "The username and password must not match!";
		return \%r;
	};

	if ( -r "/usr/local/etc/passwd.badpass" )
	{
		my @lines = MATT::Utility::ReadFile("/usr/local/etc/passwd.badpass");
		foreach my $line (@lines)
		{
			chomp $line;
			if ($pass eq $line)
			{
				$r{'error'} = "$pass is a weak password. Please select another.";
				return \%r;
			};
		};
	};
	
	$r{'rc'} = 1;
	return \%r;
};

sub DoesSysUserExist
{
	my ($user) = @_;

=head2 DoesSysUserExist

	use MATT::Passwd;
	DoesSysUserExist($user)

	$user is the username you are adding. This function merely checks to see
	if the user exists in the system password files. I use this before adding
	a new user (easy error trapping) and again after adding a user (to verify
	success). 

=cut
	if ( $user ne lc $user ) 
	{
		carp "\tDoesSysUserExist: FAILED: usernames must be lower case!\n\n";
		return 0;
	};

	if ( getpwnam($user) && getpwnam($user) > 0 )
	{
		return 1;
	} 
	else { return 0 };
};

sub Encrypt
{
	my ($pass, $debug) = @_;

=head2 Encrypt

	Encrypt ($pass, $debug)

	Encrypt (MD5) the plain text password that arrives at $pass.

=cut

	LoadModule("Crypt::PasswdMD5", "p5-Crypt-PasswdMD5", "security");

	my $salt     = rand;
	my $pass_e   = unix_md5_crypt($pass, $salt);

	print "Encrypt: pass_e = $pass_e\n" if $debug;
	return $pass_e;
};

sub BackupMasterPasswd
{
	my $file = "/etc/master.passwd";

=head2 BackupMasterPasswd

	BackupMasterPasswd ($file)

	Back up the /etc/master.passwd database. This copies $file to a new
	file named $file.nnnnnn with nnnnnn being a date/timestamp.

=cut

   
	SysCmd("sudo cp $file $file.bak");
           
#	use File::Copy;
#	copy($file, "$file.bak")
#		or carp "FATAL: Couldn't back up $file: $!\n";
};

sub VerifyMasterPasswd
{
	my ($passwd, $change, $debug) = @_;

=head2 VerifyMasterPasswd

	VerifyMasterPasswd ($passwd, $change, $debug)

	Verify that new master.passwd is the right size. I found this necessary 
	on some versions of FreeBSD as a race condition would cause the master.passwd 
	file to get corrupted. Now I verify that after I'm finished making my changes, 
	the new file is a small amount larger (or smaller) than the original.

	$passwd is the filename of your master.passwd file.
	$change is whether the file should "shrink" or "grow"

=cut

	my $new = (stat($passwd))[7];
	my $old = (stat("$passwd.bak"))[7];

	if ( $new == $old ) 
	{
		print "VerifyMasterPasswd: The files are the same size ($new)!\n" if $debug;
	} 
	elsif ( $change eq "grow" && $new > $old )
	{
		print "VerifyMasterPasswd: The file grew ($old to $new) bytes.\n" if $debug;
	} 
	elsif ( $change eq "grow" && $old > $new )
	{
		print "VerifyMasterPasswd: WARNING: The new file is smaller ($new) than $old!\n";
		print "Making archive of master.passwd...";
		ArchiveFile("$passwd.bak");
		print "done.\n";
	} 
	elsif ( $change eq "shrink" && $old > $new )
	{
		if ( ($old - $new) < 300 ) 
		{
			print "VerifyMasterPasswd: The file shrank ($old to $new) bytes.\n" if $debug;
		} 
		else 
		{
			print "VerifyMasterPasswd: WARNING: The file shrank ($old to $new) bytes.\n";
		};
	} 
	else 
	{
		print "VerifyMasterPassword: Yikes, how'd I get here?\n";
	};
};

sub InstallGroup($;$)
{
	my ($group, $gid) = @_;

=head2 InstallGroup

	InstallGroup ($group, $gid)

	Installs a system group. The $gid is optional.

=cut

	my $r = getgrnam($group);

	if ( $r )
	{
		print "InstallGroup: group $group (gid: $r) installed\n";
	}
	else
	{
		print "InstallGroup: installing $group\n";

		my $pw = FindTheBin("pw");

		if ( $gid )
		{
			SysCmd( "$pw groupadd -n $group -g $gid");
		} else {
			SysCmd( "$pw groupadd -n $group");
		};
	};
};

sub InstallUser($;$$)
{
	my ($user, $shell, $homedir) = @_;

=head2 InstallUser

	InstallUser ($user, $shell, $homedir)

	Installs a system user. The $shell and $homedir variables are optional.

=cut

	my $r  = getpwnam($user);

	$shell = "/sbin/nologin" if (!$shell);

	if ( getpwnam($user) )
	{
		print "user $user (uid: $r) already exists\n";
	}
	else
	{
		print "installing $user\n";

		my $pw = FindTheBin("pw");

		if (  $user eq "vpopmail" )
		{
			SysCmd("$pw useradd -n $user -u 89 -g 89 -c Vpopmail-Master -d $homedir -s $shell");
		}
		elsif (  $user eq "clamav" )
		{
			my $gid = getgrnam("clamav");
			SysCmd( "$pw useradd -n $user -g $gid -d /nonexistent -c Clam-Antivirus -s $shell");
		}
		else
		{
			if ($homedir)
			{
				SysCmd( "$pw useradd -n $user -d $homedir -s $shell");
			} else {
				SysCmd( "$pw useradd -n $user -s $shell");
			};
		};
	};
};

1;
__END__


=head1 AUTHOR

	Matt Simerson <matt@cadillac.net>

=head1 BUGS

	None known. Report any to author.

=head1 TODO

	Don't export any of the symbols by default. Move all symbols
	to EXPORT_OK and explicitely pull in the required ones in 
	programs that need them.

=head1 SEE ALSO

	http://www.tnpi.biz/internet/
	http://www.tnpi.biz/computing/

=head1 COPYRIGHT

	Copyright 2003, The Network People, Inc. All Rights Reserved.

=cut
