#!/usr/bin/perl

package MATT::Perl;

=head1 NAME

	MATT::Perl - Common Perl functions

=head1 SYNOPSIS

	MATT::Perl is a grouping of frequently used functions I've written

=head1 DESCRIPTION

=cut

use strict;
use Exporter;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

$VERSION = 0.6.0;

@ISA       = qw( Exporter );
@EXPORT    = qw( 
	LoadModule
	CheckPerl
	InstallPerl
);
@EXPORT_OK = qw();

use Carp;
use POSIX qw(uname);
use lib ".";
use lib "lib";
use MATT::Utility;

sub CheckPerl
{
	if ($] < 5.006001) 
	{
		warn qq{\a\a\a
**************************************************************************
**************************************************************************
  Version $] of perl is NOT SUPPORTED by several mail toaster components. 
  You should strongly consider upgrading your perl before continuing. 
  Perl version 5.6.1 is the lowest version supported.

  Press return to continue...
**************************************************************************
**************************************************************************
		};

		if ( (uname)[0] eq "FreeBSD" && -d "/usr/ports/lang/perl5" ) 
		{
			if ( YesOrNo("Would you like me to install Perl 5.6 for you") )
			{
				use MATT::FreeBSD;
				InstallPort("perl5", "lang", "perl-5.6");
				MATT::Utility::SysCmd("/usr/local/bin/use.perl port");
			};
		};
	};
};

sub LoadModule($;$$$)
{
	my ($mod, $name, $dir, $warn) = @_;

=head2 LoadModule

	LoadModule ( $mod, $name, $dir, $warn)
	LoadModule ( "CGI.pm", "p5-CGI.pm", "www")

	Loads a required Perl module. If the load fails, we attempt to install
	the required module (rather than failing gracelessly).

	$mod is the name of the module: (ie. LWP::UserAgent)
	$name is the name of the directory FreeBSD will find it in in ports
	$dir is is the ports directory where the module will be found
	$warn - if set, we warn instead of dying upon failure

=cut

	if (eval "require $mod") 
	{
		$mod->import();
	}
	else
	{
		carp "\ncouldn't import $mod: $@\n\n Would you like me to install $mod: \n";
		if ( YesOrNo() ) 
		{
			if ( $dir && (uname)[0] eq "FreeBSD" && -e "/usr/ports/$dir/$name" ) 
			{
				use MATT::FreeBSD;
				MATT::FreeBSD::InstallPort($name, $dir);
			}
			else {
				use CPAN;
				install "$mod";
			};
			$mod->import();
		} 
		else 
		{ 
			if ($warn) { carp "\n$mod is required, you have been warned.\n" }
			else       { croak "\nI'm sorry, $mod is required to continue.\n" };
		};
	};
};

sub InstallPerl
{

=head1 InstallPerl

	use MATT::Perl;
	InstallPerl("perl-5.8.1");

	currently only works on FreeBSD and Darwin (Mac OS X)

=cut

	if ( (uname)[0] eq "FreeBSD" ) 
	{
		use MATT::FreeBSD;
		MATT::FreeBSD::InstallPort ("perl5", "lang", "", "perl-5");
		MATT::Utility::SysCmd ("/usr/local/bin/use.perl port");

	}
	elsif ( (uname)[0] eq "Darwin" )
	{
		# as directed at http://developer.apple.com/internet/macosx/
		
		my ($perl) = @_;
		unless ( -d "/usr/local/src") { mkdir("/usr/local/src", 0755) };
		chdir("/usr/local/src");

		$perl = "perl-5.8.1" unless ($perl);

		unless ( -e "$perl.tar.gz" ) {
			MATT::Utility::FetchFile("ftp://ftp.perl.org/pub/CPAN/src/$perl.tar.gz");
		};

		if ( -d $perl )
		{
			my $r = MATT::Utility::SourceWarning($perl, 1);
		};
		my $tar = MATT::Utility::FindTheBin("tar");
		MATT::Utility::SysCmd("$tar -xzf $perl.tar.gz");

		chdir($perl);
		my $replace = MATT::Utility::YesOrNo("\n\n    NOTICE!   \n
Apple installs Perl in /usr/bin. I can install it there as well, overwriting Darwin's
supplied Perl (5.6.0) or it can be installed in /usr/local/bin (The BSD Way). Some
would feel that's safer as it leaves Darwin's distributed version of Perl in tact. I
would suggest that's apt to cause you grief later.\n

Shall I overwrite the default Perl?");

		if ( $replace ) { SysCmd("./Configure -de -Dprefix=/usr"); } 
		else            { SysCmd("./Configure -de");               };

		SysCmd("make");
		SysCmd("make test");

		my $install = MATT::Utility::YesOrNo("\n\n
OK, I just finished running \"make test\" and the results are show above. You should see
a success rate somewhere along the lines of 99.7% okay with only a couple failures. If that's
the case, just feed me a y and I'll install perl for you. Select n to cancel.\n\n");

		if ( $install ) 
		{
			my $sudo = "";
			if ( $< ne "0" ) { $sudo = "/usr/bin/sudo -p 'Password for %u@%h:'"; };

			if ($replace) 
			{
				# while this code isn't critical, it will fix the problem
				# posted here: http://dev.perl.org/perl5/news/2002/07/18/580ann/perldelta.html#mac%20os%20x%20dyld%20undefined%20symbols
				use File::Copy;
				my $file = "/Library/Perl/darwin/CORE/libperl.dylib";
				if ( -e $file ) {
					move($file, "$file.old") or carp "failed to remove $file\n";
				};
				$file = "/System/Library/Perl/darwin/CORE/libperl.dylib";
				if ( -e $file ) {
					move($file, "$file.old") or carp "failed to remove $file\n";
				};
				SysCmd("$sudo find /Library/Perl -name '*.bundle' -exec rm {} \;");
				SysCmd("$sudo find /System/Library/Perl -name '*.bundle' -exec rm {} \;");
			};
			SysCmd("$sudo make install");
		};

	};
};


1;
__END__


=head1 AUTHOR

	Matt Simerson <matt@cadillac.net>

=head1 BUGS

	None known. Report any to author.

=head1 TODO

=head1 SEE ALSO

	http://www.tnpi.biz/computing/
	http://www.tnpi.biz/computing/perl/MATT-Bundle/

=head1 COPYRIGHT

	Copyright 2003, The Network People, Inc. All Rights Reserved.

=cut

