#!/usr/bin/perl

package MATT::Qmail;

=head1 NAME

	MATT::Qmail - Common Qmail functions

=head1 SYNOPSIS

	MATT::Qmail is a grouping of frequently used functions I've written
	for perl use with Qmail.

=head1 DESCRIPTION

=cut

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;

@ISA     = qw(Exporter);
@EXPORT  = qw(
		StopQmail 
		StartQmail 
		CheckQmailControl 
		CheckQmailQueue 
		ProcessQmailQueue
		CheckRcpthosts
		ConfigQmail
		ConfigureQmailControl
		InstallQmailServiceLogRunFiles
		InstallQmailServiceRunFiles
	);
$VERSION = 1.2.2;

use lib ".";
use lib "lib";
use MATT::Utility;

##
# MATT::Qmail
##
# -----------------------------------------------------------------------------  

my $qdir     = "/var/qmail/queue";
my $qcontrol = "/service/send";

sub StopQmail
{

=head2 StopQmail

	StopQmail

	Use StopQmail to quit the qmail-send process. It will send qmail-send the
	TERM signal and then wait until it's shut down before returning. If qmail-send
	fails to shut down within 100 seconds, then we force kill it, causing it to
	abort any outbound SMTP sessions that are active. This is safe, as qmail will
	attempt to deliver them again, and again until it succeeds.

=cut

	my $svc    = FindTheBin("svc");
	my $svstat = FindTheBin("svstat");

	# send qmail-send a TERM signal
	system "$svc -d $qcontrol";

	# loop up to a thousand seconds waiting for qmail-send to die
	foreach my $i ( 1..1000 ) {
		my $r = `$svstat $qcontrol`;
		chomp $r;
		if ( $r =~ /^.*:\sdown\s[0-9]*\sseconds/ ) {
			print "Yay, we're down!\n";
			return 0;
		} else {
			# if more than 100 seconds passes, lets kill off the qmail-remote
			# processes that are forcing us to wait.
			
			if ($i > 100) {
				system "killall qmail-remote";
			};
			print "$r\n";
		};
		sleep 1;
	};
	return 1;
};

sub StartQmail
{

=head2 StartQmail

	StartQmail - Start up the qmail-send process.

	After starting up qmail-send, we verify that it's running before 
	returning.

=cut

	my $svc    = FindTheBin("svc");
	my $svstat = FindTheBin("svstat");

	# Start the qmail-send (and related programs)
	system "$svc -u $qcontrol";

	# loop until it's up and running.
	foreach my $i ( 1..100 ) {
		my $r = `$svstat $qcontrol`;
		chomp $r;
		if ( $r =~ /^.*:\sup\s\(pid [0-9]*\)\s[0-9]*\sseconds$/ ) {
			print "Yay, we're up!\n";
			return 0;
		};
		sleep 1;
	};
	return 1;
};

sub CheckQmailControl
{
	my $dir = shift;
	
	if ( ! -d $dir )
	{
		print "HEY! The control directory for qmail-send is not
		in $dir where I expected. Please edit this script
		and set $qcontrol to the appropriate directory!\n";
	};
};

sub CheckQmailQueue
{
	my $dir = shift;
	
	if ( ! -d $dir )
	{
		print "HEY! The queue directory for qmail is not
		$dir where I expect. Please edit this script
		and set $qdir to the appropriate directory!\n";
	};
};

sub ProcessQmailQueue
{

=head2 ProcessQmailQueue
	
	ProcessQmailQueue - Tell qmail to process the queue immediately

=cut

	my $svc    = FindTheBin("svc");

	print "\nSending ALRM signal to qmail-send.\n";
	system "$svc -a $qcontrol";
};

sub CheckRcpthosts
{
	my ($qmaildir) = @_;

	my $assign = "$qmaildir/users/assign";
	my @domains = GetDomainsFromAssign($assign);

	my $rcpt   = "$qmaildir/control/rcpthosts";
	my $mrcpt  = "$qmaildir/control/morercpthosts";

	my @f1      = ReadFile( $rcpt  );
	my @f2      = ReadFile( $mrcpt );
	my %rcpthosts;

	foreach my $f (@f1, @f2)
	{
		chomp $f;
		$rcpthosts{$f} = 1;
	};

	foreach my $v (@domains)
	{
		my $domain = $v->{'dom'};
		if ( ! $rcpthosts{$domain} )
		{
			print "$domain\n";
		};
	};

	print "\nDomains listed above should be added to the file 
$mrcpt. Don't forget to do this afterwards:

	$qmaildir/bin/qmail-newmrh
\n";
};

sub ConfigQmail 
{
	my ($qmaildir, $host, $postmaster) = @_;
	my $dbhost;
	my $control = "$qmaildir/control";

	unless ( $host ) 
	{
		$host = MATT::Utility::GetAnswer("the hostname for this mail server");
	};

	unless ( $dbhost ) 
	{
		$dbhost = MATT::Utility::GetAnswer("the hostname for your database server (localhost)");
	};
	$dbhost = "localhost" unless ($dbhost);

	unless ( $postmaster ) 
	{
		$postmaster = MATT::Utility::GetAnswer("the email address you use for administrator mail");
	};

	my $password = MATT::Utility::GetAnswer("the SQL password for user vpopmail");

	unless ( -e "$control/concurrencyremote" ) 
	{
		MATT::Utility::WriteFile("$control/concurrencyremote", "255");
		chmod(00644, "$control/concurrencyremote");
	};

	unless ( -e "$control/me" ) 
	{
		MATT::Utility::WriteFile("$control/me", $host);
	};

	unless ( -e "$control/tarpitcount" ) 
	{
		MATT::Utility::WriteFile("$control/tarpitcount", "50");
	};

	unless ( -e "$control/tarpitdelay" ) 
	{
		MATT::Utility::WriteFile("$control/tarpitdelay", "5");
	};

	unless ( -e "$qmaildir/alias/.qmail-postmaster" ) 
	{
		MATT::Utility::WriteFile("$qmaildir/alias/.qmail-postmaster",    "$postmaster\@$host");
	};

	unless ( -e "$qmaildir/alias/.qmail-root" ) 
	{
		MATT::Utility::WriteFile("$qmaildir/alias/.qmail-root",          "$postmaster\@$host");
	};

	unless ( -e "$qmaildir/alias/.qmail-mailer-daemon" ) 
	{
		MATT::Utility::WriteFile("$qmaildir/alias/.qmail-mailer-daemon", "$postmaster\@$host");
	};

	unless ( -e "$control/sql" ) 
	{
		my @lines  = "server $dbhost";
		push @lines, "port 3306";
		push @lines, "database vpopmail";
		push @lines, "table relay";
		push @lines, "user vpopmail";
		push @lines, "pass $password";
		push @lines, "time 1800";

		MATT::Utility::WriteFile("$control/sql", @lines);

		my $uid = getpwnam("vpopmail");
		my $gid = getgrnam("vchkpw");
		chown( $uid, $gid, "$control/sql"); 
		chown( $uid, $gid, "$control/servercert.pem"); 
	};
};

sub ConfigureQmailControl($$$)
{
	my ($qmaildir, $confdir, $toaster) = @_;
	
	if ( -e "$qmaildir/rc" ) 
	{
		print "ConfigureQmailControl: $qmaildir/rc already exists.\n";
	}
	else 
	{
		my @lines = "#!/bin/sh\n";

		push @lines, "# This uses splogger to send logging through syslog";
		push @lines, "exec env - PATH=\"/var/qmail/bin:\$PATH\" qmail-start ./Maildir/ splogger qmail";
		push @lines, "\n";
		push @lines, "# This sends the output to multilog as directed in log/run";
		push @lines, "#exec env - PATH=\"/var/qmail/bin:\$PATH\" qmail-start ./Maildir/";

		MATT::Utility::WriteFile("$qmaildir/rc", @lines);
		chmod(00751, "$qmaildir/rc");
	};

	if ( -e "$confdir/rc.d/qmail.sh" ) 
	{
		unlink("$confdir/rc.d/qmail.sh") 
			or die "couldn't delete $confdir/rc.d/qmail.sh: $!";
	};

	my $qmailctl  = "/usr/local/sbin/qmail";
	if ( -e $qmailctl ) 
	{
		print "ConfigureQmailControl: $qmailctl already exists.\n";
	}
	else 
	{
		MATT::Utility::FetchFile("$toaster/start/qmail.txt");
		use File::Copy;
		move("qmail.txt", $qmailctl) or die "couldn't move: $!";
		chmod(00751, $qmailctl);
		MATT::Utility::SysCmd("$qmailctl cdb");
	};
};

sub InstallQmailServiceRunFiles($$$)
{
	my ($supervise, $qmaildir, $qmuid) = @_;

	my $servername = GetAnswer("this mail servers (pop3) host name");

	foreach my $prot ( qw/ smtp send pop3 / )
	{
		my $run_f = "$supervise/$prot/run";
		
		unless ( -e  $run_f )
		{
			my $vpdir = (getpwnam("vpopmail"))[7];
			my @lines = "#!/bin/sh\n";

			if ($prot eq "smtp" or $prot eq "pop3")
			{
				push @lines, "PATH=/var/qmail/bin:/usr/local/bin:/usr/bin:/bin";
				push @lines, "export PATH\n";
			};

			if ($prot eq "smtp")
			{	
				my $qmgid = getgrnam("qnofiles");
				$qmgid = getgrnam("qmail") if (! $qmgid);
	
				push @lines, "LOCAL=\`head -1 $qmaildir/control/me\`";
				push @lines, "if [ -z \"\$LOCAL\" ]; then";
				push @lines, "\techo LOCAL is unset in";
				push @lines, "\t$qmaildir/supervise/smtp/run";
				push @lines, "\texit 1";
				push @lines, "fi\n";
	
				push @lines, "if [ ! -f $qmaildir/control/rcpthosts ]; then";
    			push @lines, "\techo \"No $qmaildir/control/rcpthosts!\"";
    			push @lines, "\techo \"Refusing to start SMTP listener because it'll create an open relay\"";
    			push @lines, "\texit 1";
				push @lines, "fi";
	
				my $exec = "exec softlimit -m 8000000 tcpserver -S -H -R -c20 -x $vpdir/etc/tcp.smtp.cdb" .
					" -u $qmuid -g $qmgid 0 smtp";
				my $bl = GetDesiredBlacklists();
				$exec = $exec . " $bl qmail-smtpd \"\$LOCAL\" $vpdir/bin/vchkpw /usr/bin/true";
				$exec = $exec . " splogger smtpd";
	
				push @lines, $exec;
				push @lines, "\n# -H don't look up \$TCPREMOTEHOST";
				push @lines, "# -R don't look up \$TCPREMOTEINFO";
				push @lines, "# -p paranoid, do forward & reverse queries";
				push @lines, "#    (if you use the -p option, consider the -t option).";
				push @lines, "# -S http://matt.simerson.net/computing/mail/qmail/qmail.toaster.open-smtp_writeup.txt";
				push @lines, "# -t timout value for dns lookups (in seconds) - default 26 seconds";
				push @lines, "# -c number of simultaneous connections";
			}
			elsif ($prot eq "send")
			{
				push @lines, "exec /var/qmail/rc";
			}
			elsif ($prot eq "pop3")
			{
				my $exec = "exec tcpserver -R -H -c100 0 pop3 qmail-popup $servername $vpdir/bin/vchkpw qmail-pop3d Maildir splogger pop3";
				push @lines, $exec;
			};
			print "InstallQmailServiceRunFiles: writing $run_f\n";
			MATT::Utility::WriteFile($run_f, @lines);
			chmod(00751, $run_f);
		};
	};
};

sub InstallQmailServiceLogRunFiles(;$$$$)
{
	my ($maillogs, $isoqlog, $services, $supervise ) = @_;
	my (@lines);

	unless ( $services and $services->[0] ne "" ) 
	{ 
		my @array = qw/ smtp send pop3 /;
		$services = \@array;
	};

	unless ( $supervise ) { $supervise = "/var/qmail/supervise" };

	# Create log/run files
	foreach my $serv ( @$services )
	{
		my $run_f   = "$supervise/$serv/log/run";

		unless ( -s $run_f ) 
		{
			print "InstallQmailServiceLogRun: creating file $run_f\n";

			     @lines= "#!/bin/sh\n\n";
			push @lines, "PATH=/var/qmail/bin:/usr/local/bin:/usr/bin:/bin\n";
			push @lines, "export PATH\n\n";

			my $runline = "exec setuidgid qmaill multilog t ";

			if ($serv eq "smtp") 
			{
				if ( $maillogs ) { $runline = $runline . "!./smtplog " };
				$runline = $runline . "s100000 /var/log/mail/smtp\n";
			} 
			elsif ( $serv eq "send") 
			{
				if ( $maillogs ) { $runline = $runline . "!./sendlog " };
				if ( $isoqlog ) { $runline = $runline . "n288 " };
				$runline = $runline . "s100000 /var/log/mail/send\n";
			} 
			elsif ( $serv eq "pop3") 
			{
				if ( $maillogs ) { $runline = $runline . "!./pop3log " };
				$runline = $runline . "s100000 /var/log/mail/pop3\n";
			};

			push @lines, $runline;
			
			MATT::Utility::WriteFile($run_f, @lines);
			chmod(00751, $run_f);
		} 
		else 
		{
			print "InstallQmailLogRun: $run_f already exists.\n";
		};
	};
};

sub GetDesiredBlacklists
{
	my @blacklists = 
		qw/ 
			list.dsbl.org sbl.spamhaus.org bl.spamcop.net relays.ordb.org 
			rbl-plus.mail-abuse.org blackholes.mail-abuse.org 
			relays.mail-abuse.org dialups.mail-abuse.org
   			korea.services.net formmail.relays.monkeys.com proxies.relays.monkeys.com
			cn.rbl.cluecentral.net kr.rbl.cluecentral.net
		/;

	print "\nDNS based blacklists:\n\n\t";
	print join("\n\t", @blacklists); 

	my $enabled = "";
	if ( MATT::Utility::YesOrNo("\n\nDo you want to enable any of the listed blacklists? ") )
	{
		$enabled = " rblsmtpd";
		foreach my $bl ( @blacklists ) 
		{
			if ( MATT::Utility::YesOrNo("enable $bl ") )
			{
				$enabled = $enabled . " -r $bl";
			};
		};
	};
	return $enabled;
};


1;
__END__


=head1 AUTHOR

	Matt Simerson <matt@cadillac.net>

=head1 BUGS

	None known. Report any to author.

=head1 TODO

=head1 SEE ALSO

	http://www.tnpi.biz/computing/
	http://www.tnpi.biz/internet/

=head1 COPYRIGHT

	Copyright 2003, The Network People, Inc. All Rights Reserved.

=cut
